/*
    Copyright (C) 1995-2000 AROS - The Amiga Research OS
    $Id: changetaskpri.c,v 1.1.1.1 2005/03/15 16:01:02 laire Exp $
 
    Desc: Change the priority of a task.
    Lang: english
*/

/**************************************************************************
 
    NAME
	ChangeTaskPri
 
    FORMAT
	ChangeTaskPri <priority> [ PROCESS <process number> ]
 
    SYNOPSIS
	PRI=PRIORITY/A/N,PROCESS/K/N
 
    LOCATION
	Workbench:c
 
    FUNCTION
	The ChangeTaskPri command is used to change the current run 
	priority of a Task. As AROS is a multitasking operating
	system, you can determine which tasks receive more CPU time
	by changing their priorities.
 
	The value of |priority| can be from -128 to 127, however values
	greater than 4 are not recommended as they can interfere with
	vital system processes. Higher values will give tasks a higher
	CPU priority.
 
	You can use the Status command to examine the list of Tasks that
	are running and their process numbers.
 
    EXAMPLE
	
	1.SYS:> ChangeTaskPri 1 Process 1
 
	    Set the priority of the current process to 1.
 
	1.SYS:> ChangeTaskPri 1
 
	    Also sets the priority of the current process to 1.
 
    SEE ALSO
	Status
 
**************************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <exec/tasks.h>
#include <exec/execbase.h>
#include <dos/dosextens.h>
#include <dos/exall.h>
#include <dos/rdargs.h>
#include <utility/tagitem.h>

#include <proto/exec.h>
#include <proto/dos.h>

#define ARG_TEMPLATE	"PRI=PRIORITY/A/N,PROCESS/K/N"
#define ARG_PRI		0
#define ARG_PROCESS	1
#define TOTAL_ARGS	2

#include "changetaskpri_version.h"

static const char version[] = VERSTAG;

static const char exthelp[] =
    "ChangeTaskPri : Change the priority of a CLI task\n"
    "\tPRI=PRIORITY/A/N	     New priority of task\n"
    "\tPROCESS/K             Optional process number of change\n";

int main(void)
{
    struct ExecBase *SysBase=*((struct ExecBase**) 4);
    struct DosLibrary	*DOSBase;
    struct Process *pr = NULL;
    struct RDArgs *rda = NULL, *rdargs;
    IPTR args[TOTAL_ARGS] = { 0, 0 };
    LONG Pri;
    int error = 0;
    int ReturnCode = 0;

    if ((DOSBase=(struct DosLibrary *)OpenLibrary("dos.library",37))!=0)
    {
        rda = AllocDosObject(DOS_RDARGS, NULL);
        if( rda != NULL )
        {
            rda->RDA_ExtHelp = (STRPTR)exthelp;

            rdargs = ReadArgs(ARG_TEMPLATE, (IPTR *)args, rda);
            if( rdargs != NULL )
            {
                Forbid();
                if( args[ARG_PROCESS] )
                {
                    pr = FindCliProc(*((ULONG*)args[ARG_PROCESS]));
                    if (!pr)
                    {
                        if ((SysBase->LibNode.lib_Version == 50 && SysBase->LibNode.lib_Revision >= 45) ||
                             SysBase->LibNode.lib_Version > 50)
                        {
                            pr = (struct Process *) FindTaskByPID(*((ULONG*) args[ARG_PROCESS]));
                        }
                    }
                }
                else
                    pr = (struct Process *)FindTask(NULL);

                if( pr != NULL )
                {
                    /* Check the bounds on the priority */
                    Pri = *((LONG*) args[ARG_PRI]);
                    if( Pri < -128 || Pri > 127 )
                        error = ERROR_OBJECT_TOO_LARGE;
                    else
                        /* Set the priority */
                        SetTaskPri( (struct Task *)pr, Pri );

                    Permit();
                }
                else
                {
                    BPTR errStream;

                    Permit();

                    errStream = Output();

                    pr = (struct Process *)FindTask(NULL);
                    if( pr->pr_CES != NULL )
                        errStream = pr->pr_CES;

                    VFPrintf(errStream, "ChangeTaskPri: Process %ld does not exist.\n", (APTR) args[ARG_PROCESS]);
                    SetIoErr(0);
                    error = -1;
                }
                FreeArgs(rdargs);
            } /* ReadArgs() ok */
            else
                error = IoErr();

            FreeDosObject(DOS_RDARGS, rda);
        } /* Got a RDArgs * */
        else
            error = IoErr();

        if( error != -1 && error != 0 )
        {
            PrintFault(error, "ChangeTaskPri");
            ReturnCode=RETURN_FAIL;
        }
        else
        {
            SetIoErr(0);
        }
        CloseLibrary((struct Library *)DOSBase);
    }
    return(ReturnCode);
}
